from datetime import datetime
from typing import IO, NotRequired, TypedDict, Unpack

from requests import Response
from .common import RESTConnector


class PaperlessPostDocumentOptions(TypedDict):
    title: NotRequired[str]
    created: NotRequired[datetime]
    correspondent: NotRequired[str]
    document_type: NotRequired[str]
    tags: NotRequired[list[str]]
    archive_serial_number: NotRequired[int]


class PaperlessConnection(RESTConnector):
    url: str
    port: int

    _token: str | None

    def __init__(
        self,
        url: str,
        port: int,
        user: str,
        password: str,
    ) -> None:
        self.url = url
        self.port = port
        token_response = self.post(
            "/api/token/",
            {
                "username": user,
                "password": password,
            },
        )

        assert token_response.ok, token_response

        self._token = token_response.json()["token"]

    def post_document(
        self, document: IO, **kwargs: Unpack[PaperlessPostDocumentOptions]
    ) -> Response:
        return self.post(
            "/api/documents/post_document/",
            files={"document": document},
            data=dict(**kwargs),
            header={"Authorization": f"Token {self._token}"},
        )

    @property
    def connected(self) -> bool:
        return bool(self._token)

    def connect(self) -> None:
        pass

    def disconnect(self) -> None:
        pass
